"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const apollo_link_1 = require("apollo-link");
class PusherLink extends apollo_link_1.ApolloLink {
    constructor(options) {
        super();
        this.pusher = options.pusher;
    }
    request(operation, forward) {
        return new apollo_link_1.Observable(observer => {
            forward(operation).subscribe({
                next: data => {
                    const subscriptionChannel = this._getChannel({ data, operation });
                    if (subscriptionChannel) {
                        this._createSubscription(subscriptionChannel, observer);
                    }
                    else {
                        observer.next(data);
                        observer.complete();
                    }
                },
            });
        });
    }
    _getChannel({ data, operation }) {
        return !!data.extensions
            && !!data.extensions.lighthouse_subscriptions
            && !!data.extensions.lighthouse_subscriptions.channels
            ? data.extensions.lighthouse_subscriptions.channels[operation.operationName]
            : null;
    }
    _createSubscription(subscriptionChannel, observer) {
        const pusherChannel = this.pusher.subscribe(subscriptionChannel);
        pusherChannel.bind('lighthouse-subscription', (payload) => {
            if (!payload.more) {
                this.pusher.unsubscribe(subscriptionChannel);
                observer.complete();
            }
            const { result } = payload;
            if (result) {
                observer.next(result);
            }
        });
    }
}
exports.default = PusherLink;
//# sourceMappingURL=PusherLink.js.map